from paircd.reply import (
    ERR_NOSUCHCHANNEL,
    ERR_NOTREGISTERED,
    JOIN,
    RPL_ENDOFNAMES,
    RPL_NAMREPLY,
    RPL_TOPIC,
)

from paircd.client import Client
from paircd.command_handler import CommandHandler
from paircd.message import Message
from paircd.server import Server


class JoinHandler(CommandHandler):
    def __init__(self) -> None:
        super().__init__("JOIN", 1)

    async def handle(self, server: Server, client: Client, msg: Message) -> None:
        channel_name = msg.args[0]
        if not channel_name.startswith("#"):
            client.write_message(ERR_NOSUCHCHANNEL(client.nickname, channel_name))
            return

        channel = server.get_channel_by_name(channel_name)
        channel.add_client(client)

        client.channels.add(channel_name)
        client.log(f"joined {channel_name}")

        channel.write_message(JOIN(channel_name, prefix=client.id()))

        if channel.topic != "":
            client.write_message(
                RPL_TOPIC(client.nickname, channel_name, channel.topic)
            )

        channel_members = [
            f"{''.join(channel.get_user_modes_by_nick(member))}{member}"
            for member in channel.clients_by_nick.keys()
        ]
        client.write_message(
            # "=" means public channel (ref: https://modern.ircdocs.horse/#rplnamreply-353)
            RPL_NAMREPLY(client.nickname, "=", channel_name, " ".join(channel_members))
        )
        client.write_message(RPL_ENDOFNAMES(client.nickname, channel_name))
